from resource_management import *
from resource_management.libraries.functions.curl_krb_request import curl_krb_request
from resource_management.core.logger import Logger
from resource_management.libraries.functions.format import format
from resource_management.core.exceptions import Fail
from requests import HTTPError
import json

try:
    import urllib.request as urllib_request
    import urllib.error as urllib_error
except ImportError:
    import urllib2 as urllib_request
    import urllib2 as urllib_error

import time

class HttpFS_service_check(Script):
    def service_check(self, env):
        import params
        env.set_params(params)
        Logger.info("Checking HttpFS service")

        http_code = 's' if params.ssl_enabled else ''

        uri = format("http{http_code}://{params.httpfs_server_host}:{params.httpfs_server_port}/webhdfs/v1?op=liststatus&user.name=httpfs")

        if params.security_enabled:
            response, errmsg, time_millis = curl_krb_request(params.tmp_dir, params.smoke_user_keytab,
                                                             params.smoke_user_principal, uri, "httpfs_service_check",
                                                             params.kinit_path_local, False, None, params.smoke_user)

            try:
                if "FileStatuses" not in response:
                    Logger.error("Response not as expected, service not healthy")
                    raise Fail("Service check failed on {0}".format(uri))

            except (HTTPError, ValueError, json.JSONDecodeError) as e:
                Logger.error("Error connecting to WEB UI on: {0}. Error: {1} Exception: {2}".format(uri, errmsg, str(e)) )
                raise Fail("Error connecting to {0}".format(uri))

        else:
            start_time = time.time()
            try:
                response = urllib_request.urlopen(uri)
                time_millis = (time.time() - start_time) * 1000
                errmsg = None
            except (urllib_error.HTTPError, urllib_error.URLError) as e:
                response = None
                errmsg = str(e) if isinstance(e, urllib_error.URLError) else e.read().decode('utf-8')
                time_millis = (time.time() - start_time) * 1000

            if response == None or response.getcode() != 200:
                Logger.error("Cannot access HttpFS on: {0}. Error : {1}".format(uri, errmsg))
                raise Fail("Error connecting to {0}".format(uri))

        Logger.info("Service working at {0}".format(uri))
        Logger.info("Service check Sucessfull")
        Logger.info("Checking HttpFS service ({0}ms) - DONE".format(time_millis))

if __name__ == "__main__":
    HttpFS_service_check().execute()